@@ -78,7 +78,7 @@ class Custodian(object):
     def __init__(self, handlers, jobs, max_errors=1, polling_time_step=10,
                  monitor_freq=30, log_file="custodian.json",
                  skip_over_errors=False, scratch_dir=None,
-                 gzipped_output=False, checkpoint=True):
+                 gzipped_output=False, checkpoint=False):
         """
         Args:
             handlers:
@@ -121,7 +121,9 @@ class Custodian(object):
                 Whether to gzip the final output to save space. Defaults to
                 False.
             checkpoint:
-                Whether to checkpoint after each Job. Defaults to True.
+                Whether to checkpoint after each successful Job.
+                Checkpoints are stored as custodian.chk.#.tar.gz
+                files. Defaults to False.
         """
         self.max_errors = max_errors
         self.jobs = jobs
@@ -135,14 +137,21 @@ class Custodian(object):
         self.checkpoint = checkpoint
 
     @staticmethod
-    def _load_checkpoint(fname):
-        logging.info("Loading from checkpoint file {}...".format(fname))
-        t = tarfile.open(fname)
-        t.extractall()
-        #Log the corrections to a json file.
-        with open(Custodian.LOG_FILE, "r") as f:
-            run_log = json.load(f)
-        return run_log
+    def _load_checkpoint(cwd):
+        restart = -1
+        run_log = []
+        for chkpt in glob(pjoin(cwd, "custodian.chk.*.tar.gz")):
+            jobno = int(chkpt.split(".")[-3])
+            if jobno > restart:
+                restart = jobno
+                logging.info("Loading from checkpoint file {}...".format(
+                    chkpt))
+                t = tarfile.open(chkpt)
+                t.extractall()
+                #Log the corrections to a json file.
+                with open(Custodian.LOG_FILE, "r") as f:
+                    run_log = json.load(f)
+        return restart, run_log
 
     @staticmethod
     def _delete_checkpoints(cwd):
@@ -176,16 +185,19 @@ class Custodian(object):
             os.chdir(tempdir)
             logging.info("Using scratch directory {}.".format(tempdir))
 
-        run_log = []
         total_errors = 0
         unrecoverable = False
         start = datetime.datetime.now()
         logging.info("Run started at {}.".format(start))
 
+        if self.checkpoint:
+            restart, run_log = Custodian._load_checkpoint(cwd)
+        else:
+            restart, run_log = -1, []
+
         for i, job in enumerate(self.jobs):
-            chk_fname = pjoin(cwd, "custodian.chk.{}.tar.gz".format(i))
-            if self.checkpoint and os.path.exists(chk_fname):
-                run_log = Custodian._load_checkpoint(chk_fname)
+            if i <= restart:
+                #Skip all jobs until the restart point.
                 continue
             run_log.append({"job": job.to_dict, "corrections": []})
             for attempt in xrange(self.max_errors):
@@ -272,12 +284,6 @@ class Custodian(object):
         end = datetime.datetime.now()
         logging.info("Run ended at {}.".format(end))
         run_time = end - start
-        if total_errors >= self.max_errors:
-            logging.info("Max {} errors reached. Exited..."
-                         .format(self.max_errors))
-        elif not unrecoverable:
-            #Cleanup checkpoint files (if any) if run is successful.
-            Custodian._delete_checkpoints(cwd)
 
         logging.info("Run completed. Total time taken = {}.".format(run_time))
 
@@ -290,6 +296,13 @@ class Custodian(object):
             shutil.rmtree(tempdir)
             os.chdir(cwd)
 
+        if total_errors >= self.max_errors:
+            logging.info("Max {} errors reached. Exited..."
+                         .format(self.max_errors))
+        elif not unrecoverable:
+            #Cleanup checkpoint files (if any) if run is successful.
+            Custodian._delete_checkpoints(cwd)
+
         return run_log
 
 
