@@ -18,6 +18,7 @@ __date__ = "2/4/13"
 
 import os
 import time
+import datetime
 import operator
 
 from custodian.custodian import ErrorHandler, backup
@@ -65,6 +66,7 @@ class VaspErrorHandler(ErrorHandler, MSONable):
 
     def __init__(self, output_filename="vasp.out"):
         self.output_filename = output_filename
+        self.errors = set()
 
     def check(self):
         self.errors = set()
@@ -94,6 +96,12 @@ class VaspErrorHandler(ErrorHandler, MSONable):
         if "brmix" in self.errors or "zpotrf" in self.errors:
             actions.append({"dict": "INCAR",
                             "action": {"_set": {"ISYM": 0}}})
+            # Based on VASP forum's recommendation, you should delete the
+            # CHGCAR and WAVECAR when dealing with these errors.
+            actions.append({"file": "CHGCAR",
+                            "action": {"_file_delete": {'mode': "actual"}}})
+            actions.append({"file": "WAVECAR",
+                            "action": {"_file_delete": {'mode': "actual"}}})
 
         if "subspacematrix" in self.errors or "rspher" in self.errors or \
                         "real_optlay" in self.errors:
@@ -172,11 +180,14 @@ class VaspErrorHandler(ErrorHandler, MSONable):
                                         "action": {"_set": changes_dict}})
                         break
 
-        m = Modder()
+        m = Modder(actions=[DictActions, FileActions])
         modified = []
         for a in actions:
-            modified.append(a["dict"])
-            vi[a["dict"]] = m.modify_object(a["action"], vi[a["dict"]])
+            if "dict" in a:
+                modified.append(a["dict"])
+                vi[a["dict"]] = m.modify_object(a["action"], vi[a["dict"]])
+            elif "file" in a:
+                m.modify(a["action"], a["file"])
         for f in modified:
             vi[f].write_file(f)
         return {"errors": list(self.errors), "actions": actions}
@@ -480,7 +491,7 @@ class NonConvergingErrorHandler(ErrorHandler, MSONable):
             return {"errors": ["Non-converging job"], "actions": None}
 
     def __str__(self):
-        return "Run not converging."
+        return "NonConvergingErrorHandler"
 
     @property
     def is_monitor(self):
@@ -502,3 +513,66 @@ class NonConvergingErrorHandler(ErrorHandler, MSONable):
                        change_algo=d.get("change_algo", False))
         else:
             return cls(output_filename=d["output_filename"])
+
+
+class PBSWalltimeHandler(ErrorHandler):
+    """
+    Check if a run is nearing the walltime of a PBS queuing system. If so,
+    write a STOPCAR with LSTOP=.True.. The PBS_WALLTIME must be in the
+    environment.
+    """
+
+    def __init__(self):
+        self.start_time = datetime.datetime.now()
+
+    def check(self):
+        if "PBS_WALLTIME" in os.environ:
+            wall_time = int(os.environ["PBS_WALLTIME"])
+            run_time = datetime.datetime.now() - self.start_time
+            total_secs = run_time.seconds + run_time.days * 3600 * 24
+            try:
+                #Intelligently determine time per ionic step.
+                o = Oszicar("OSZICAR")
+                nsteps = len(o.ionic_steps)
+                time_per_step = total_secs / nsteps
+            except Exception as ex:
+                time_per_step = 0
+
+            # If the remaining time is less than average time for 3 ionic
+            # steps.
+            if wall_time - total_secs < time_per_step * 3:
+                return True
+        return False
+
+    def correct(self):
+        #Write STOPCAR
+        actions = [{"file": "STOPCAR",
+                    "action": {"_file_create": {'content': "LSTOP = .TRUE."}}}]
+        m = Modder(actions=[FileActions])
+        for a in actions:
+            m.modify(a["action"], a["file"])
+        # Actions is being returned as None so that the job will stop after
+        # STOPCAR is written.
+        return {"errors": ["Walltime reached"], "actions": None}
+
+    def __str__(self):
+        return "PBSWalltimeHandler."
+
+    @property
+    def is_monitor(self):
+        return True
+
+    @property
+    def is_terminating(self):
+        # The PBS handler should not terminate as we want VASP to terminate
+        # itself naturally with the STOPCAR.
+        return False
+
+    @property
+    def to_dict(self):
+        return {"@module": self.__class__.__module__,
+                "@class": self.__class__.__name__}
+
+    @classmethod
+    def from_dict(cls, d):
+        return cls()
